<?php

class CancellationFilter
{
    private $filters;

    public function __construct($filters)
    {
        $this->filters = $filters;
    }

    public function process($query)
    {
        $query
            ->innerJoin('Request.Listing l')
            ->innerJoin('Request.Driver d')
            ->innerJoin('l.Owner o')
            ->innerJoin('l.PickupLocation pl')
            ->innerJoin('l.DropoffLocation dl')
            ->addWhere('(Request.status in (?, ?, ?) or Request.no_show_fee = 1)', array(
                Request::STATUS_DRIVER_CANCELED,
                Request::STATUS_OWNER_CANCELED,
                Request::STATUS_CANCELED_BY_DRIVER
            ))
            ->addWhere('Request.when_accepted is not null')
            ->addWhere('l.accepted_request_id is not null')
        ;

        $this
            ->addCountryFilter($query)
            ->addStatusFilter($query)
            ->addRefundStatusFilter($query)
            ->addFeeStatusFilter($query)
            ->addTypeFilter($query)
            ->addTCIDFilter($query)
            ->addUserFilter($query)
        ;
    }

    private function addCountryFilter($query)
    {
        if (isset($this->filters['country']) && $this->filters['country'] !== '')
        {
          $query->addWhere("l.PickupLocation.Category.country_id = ?", $this->filters['country']);
        }

        return $this;
    }

    private function addStatusFilter($query)
    {
        if (isset($this->filters['status']) && $this->filters['status'] === 'closed')
        {
            $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CLOSED);
        }
        elseif (!isset($this->filters['status']) || $this->filters['status'] === 'opened')
        {
            $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CLOSED, true);
        }

        return $this;
    }

    private function addRefundStatusFilter($query)
    {
        if (isset($this->filters['refund_status']) && $this->filters['refund_status'] !== '')
        {
            switch($this->filters['refund_status'])
            {
                case 'refunded':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_REFUNDED);
                    break;
                case 'not_refunded':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NOT_REFUNDED);
                    break;
                case 'refund_requested':
                    $query->addWhere('(Request.no_show_fee = 1 or Request.status in (?, ?))', [Request::STATUS_DRIVER_CANCELED, Request::STATUS_CANCELED_BY_DRIVER]);
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_REFUNDED, true)
                         ->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NOT_REFUNDED, true)
                    ;
                    break;
                case 'na':
                    $query->addWhere('Request.status = ?', Request::STATUS_OWNER_CANCELED);
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_REFUNDED, true)
                         ->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NOT_REFUNDED, true)
                    ;

                    break;
            }
        }

        return $this;
    }

    private function addFeeStatusFilter($query)
    {
        if (isset($this->filters['fee_status']) && $this->filters['fee_status'] !== '')
        {
            switch($this->filters['fee_status'])
            {

                case 'charged':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CHARGED);
                    break;
                case 'failed':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CC_FAILED);
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CHARGED, true);
                    break;
                case 'not_charged':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NOT_CHARGED);
                    break;
                case 'fee_available':
                    $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_CHARGED, true)
                         ->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NO_SHOW|Request::INTERNAL_STATUS_LATE_CANCELLATION)
                    ;

                    $query
                          ->addWhere('((Request.status in (?,?) and is_late_cancel = 1) OR no_show_fee = 1)', [Request::STATUS_DRIVER_CANCELED, Request::STATUS_CANCELED_BY_DRIVER])
                    ;
                    break;
                case 'na':
                    $query->addWhere('(Request.no_show_fee != 1)');
                    // $this->addInternalStatusQuery($query, Request::INTERNAL_STATUS_NO_SHOW|Request::INTERNAL_STATUS_LATE_CANCELLATION, true);

                    $query->addWhere('((Request.status = ?) OR (Request.internal_status & ? && Request.internal_status & ?) OR (Request.status in (?, ?) and (Request.no_show_fee != 1 and Request.is_late_cancel != 1 and Request.internal_status & 12)))', [
                        Request::STATUS_OWNER_CANCELED,
                        Request::INTERNAL_STATUS_CHARGED|Request::INTERNAL_STATUS_NOT_CHARGED,
                        Request::INTERNAL_STATUS_REFUNDED|Request::INTERNAL_STATUS_NOT_REFUNDED,
                        Request::STATUS_DRIVER_CANCELED,
                        Request::STATUS_CANCELED_BY_DRIVER
                    ]);
                    break;
            }
        }

        return $this;
    }

    private function addTypeFilter($query)
    {
        if (isset($this->filters['type']) && $this->filters['type'] !== '')
        {
            switch ($this->filters['type']) {
                case Request::STATUS_OWNER_CANCELED:
                case Request::STATUS_DRIVER_CANCELED:
                case Request::STATUS_CANCELED_BY_DRIVER:
                    $query->addWhere('Request.status = ?', $this->filters['type']);
                    break;

                case 'no_show':
                    $query->addWhere('Request.no_show_fee = 1');
                    break;
            }
        }

        return $this;
    }

    private function addTCIDFilter($query)
    {
        if (isset($this->filters['tcid']) && $this->filters['tcid'] !== '')
        {
            $query->addWhere('Request.listing_id = ?', intval($this->filters['tcid']));
        }

        return $this;
    }

    private function addUserFilter($query)
    {
        if (isset($this->filters['user']) && $this->filters['user'] !== '' && $this->filters['user'] !== 'all')
        {
            list($type, $id) = explode('_', $this->filters['user']);
            if ('driver' == $type)
            {
               $query->addWhere('Request.driver_id = ?', $id);
            }
            else
            {
                $query->addWhere('l.owner_id = ?', $id);
            }
        }

        return $this;
    }

    private function addInternalStatusQuery($query, $status, $not = false)
    {
        $query->addWhere(($not ? 'NOT ' : '').'(Request.internal_status & ?)', $status);

        return $this;
    }
}