<?php

use Transfercar\Exception\CreditCardException;
use Transfercar\Mailchimp\MailchimpApi;

class AddRequestForm extends BaseForm
{
    private $paymentForm, $userDetailsForm;

    public function configure()
    {
        $sms_confirmed = $this->getOption('user')->get('sms_number_confirmed');
        $this->widgetSchema['checksum']     = new sfWidgetFormInputHidden();
        $this->widgetSchema['is_resubmit']  = new sfWidgetFormInputHidden();
        $this->widgetSchema['pickup_date']  = new sfWidgetFormChoice(array('choices' => $this->getAvailableDates()));
        $this->widgetSchema['dropoff_date'] = new sfWidgetFormChoice(array('choices' => $this->getAvailableDates()));
        $this->widgetSchema['pickup_time']  = new sfWidgetFormChoice(array('choices' => $this->getPickupTimes()));
        $this->widgetSchema['dropoff_time'] = new sfWidgetFormChoice(array('choices' => $this->getDropoffTimes()));
        $this->widgetSchema['email_newsletter'] = new sfWidgetFormInputCheckbox(array(), array());
        $this->widgetSchema['email_notify_new_reloc'] = new sfWidgetFormInputCheckbox(array('value_attribute_value' => 'daily'), array());

        if ($sms_confirmed)
        {
            $this->widgetSchema['sms_notify_update_req'] = new sfWidgetFormInputCheckbox(array(), array());
            $this->validatorSchema['sms_notify_update_req'] = new sfValidatorBoolean(array("required" => false));
            $this->setDefault('sms_notify_update_req', $this->getOption('user')->get('sms_notify_update_req'));
        }

        $this->validatorSchema['pickup_date'] = new sfValidatorDate(array(
            'date_format' => '~(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})~'
        ));

        $this->validatorSchema['dropoff_date'] = new sfValidatorDate(array(
            'date_format' => '~(?P<year>\d{4})-(?P<month>\d{2})-(?P<day>\d{2})~'
        ));

        $this->validatorSchema['pickup_time']  = new sfValidatorInteger(array('required' => false));
        $this->validatorSchema['dropoff_time'] = new sfValidatorInteger(array('required' => false));
        $this->validatorSchema['checksum']     = new sfValidatorChoice(array(
            'choices' => array(
                $this->getOption('listing')->getChecksum()
            )),
            array('invalid' => 'The rental operator company has updated the listing, please go back and make a new request')
        );
        $this->validatorSchema['is_resubmit'] = new sfValidatorBoolean(array('required' => false));

        $this->validatorSchema['email_newsletter'] = new sfValidatorBoolean(array("required" => false));
        $this->validatorSchema['email_notify_new_reloc'] = new sfValidatorChoice(array('choices' => array('daily'), "required" => false));

        $this->widgetSchema->setNameFormat('request[%s]');
        $this->setDefault('email_newsletter', $this->getOption('user')->get('email_newsletter'));
        $this->setDefault('email_notify_new_reloc', ($n = $this->getOption('user')->get('email_notify_new_reloc')) == 'none' ? null : $n);


        $this->mergePostValidator(new sfValidatorSchemaCompare(
            'pickup_date',
            sfValidatorSchemaCompare::LESS_THAN_EQUAL,
            'dropoff_date',
            array(),
            array(
                'invalid' => 'Dropoff date can\'t be before the pickup date'
            )
        ));

        $this->mergePreValidator(new sfValidatorCallback(array('callback' => array($this, 'addTimeValidator'))));

        $this->mergePostValidator(new tcRequestedDateValidator(array(
            'listing' => $this->getOption('listing')
        )));

        /*
            requested times validator must be after tcRequestedDateValidator
            because we need free_days and paid_days values
        */
        $this->mergePostValidator(new tcRequestedTimesValidator(array(
            'listing' => $this->getOption('listing')
        )));

        $this->mergePaymentForm();
        $this->mergeUserDetailsForm();

        $this->setDefault('checksum', $this->getOption('listing')->getChecksum());
    }

    public function addTimeValidator($validator, $values)
    {
        if ($values['pickup_date'] == $values['dropoff_date'])
        {
            $this->mergePostValidator(new sfValidatorSchemaCompare(
                'dropoff_time',
                sfValidatorSchemaCompare::GREATER_THAN_EQUAL,
                'pickup_time',
                array(),
                array(
                    'invalid' => 'Dropoff time can\'t be before the pickup time'
                )
            ));
        }

        return $values;
    }

    private function getAvailableDates()
    {
        sfLoader::loadHelpers(array('Date', 'Transfercar'));
        $listing = $this->getOption('listing');

        $period = new DatePeriod(
            new DateTime(date('Y-m-d', $listing->getMinimumPickupTimestamp())),
            new DateInterval('P1D'),
            (new DateTime($listing->get('date_return_by')))->modify('+1 day')
        );

        $range = [];
        foreach ($period as $period) {
            $range[$period->format('Y-m-d')] = tc_format_date($period->format('Y-m-d'), 'short');
        }

        return $range;
    }

    private function getPickupTimes()
    {
        $listing = $this->getOption('listing');

        $earliest = $listing->get('pickup_time');
        $latest = $listing->get('latest_pickup_time');

        if (!$this->getDefault('pickup_time'))
        {
            $this->setDefault('pickup_time', max($earliest, $listing->get('dropoff_time')));
        }

        return $this->getTimes((int) $earliest, (int) $latest);
    }

    private function getDropoffTimes()
    {
        $listing = $this->getOption('listing');

        $earliest = $listing->get('dropoff_time');
        $latest = $listing->get('latest_dropoff_time');

        if (!$this->getDefault('dropoff_time'))
        {
            $this->setDefault('dropoff_time', max($earliest, $listing->get('dropoff_time')));
        }

        return $this->getTimes((int) $earliest, (int) $latest);
    }

    private function getTimes($earliest, $latest)
    {
      sfLoader::loadHelpers(array('Date', 'Transfercar'));
      $earliest = $earliest ? $earliest : 34200;
      $latest = $latest ? $latest : 59400;
      $time  = strtotime('today');
      $times = array('' => '');

      for ($seconds = $earliest; $seconds <= $latest; $seconds+=1800)
      {
        $times[$seconds] = tc_format_date($seconds, 'hours');
      }

      return $times;
    }

    private function parseTime($time)
    {
        $hours = floor($time / 3600);
        $minutes = round(($time % 3600)/60);

        return array($hours, $minutes);
    }

    private function setDaysDefaults()
    {
        list($pickup_hour, $pickup_minutes) = $this->parseTime($this->getDefault('pickup_time'));
        list($dropoff_hour, $dropoff_minutes) = $this->parseTime($this->getDefault('dropoff_time'));

        $listing = $this->getOption('listing');
        $date = strtotime(sprintf('%s %s:%s:00', $this->getDefault('pickup_date'), $pickup_hour, $pickup_minutes));
        $end_ts = strtotime(sprintf('%s %s:%s:00', $this->getDefault('dropoff_date'), $dropoff_hour, $dropoff_minutes));
        list($free_days, $paid_days) = $listing->getDaysFor($date, $end_ts);

        $this->setDefault('free_days', $free_days);
        $this->setDefault('paid_days', $paid_days);
    }

    public function getRequest()
    {
        $listing = $this->getOption('listing');
        $method = $this->isValid() ? 'getValue' : 'getDefault';

        if (!$this->isBound()) {
            $this->setDaysDefaults();
        }

        $request = new Request();
        $request->listing_id = $listing->id;
        $request->status = Request::STATUS_PENDING;
        $request->driver_id = $this->getOption('user')->get('id');
        $request->pickup_date = $this->$method('pickup_date');
        $request->drop_off_date = $this->$method('dropoff_date');
        $request->pickup_time = $this->$method('pickup_time');
        $request->dropoff_time = $this->$method('dropoff_time');
        $request->nb_free_days = $this->$method('free_days');
        $request->nb_paid_days = $this->$method('paid_days');
        $request->is_resubmit = $this->$method('is_resubmit');

        return $request;
    }

    public function save()
    {
        $listing = $this->getOption('listing');

        try {
            $conn = Doctrine_Manager::connection();
            $conn->beginTransaction();

            $request = $this->getRequest();
            $request->created_at = date('Y-m-d H:i:s');
            $request->setDataFromListing($listing);

            $this->saveUser();
            $this->saveCreditCard($request);
            $this->saveUserNotificationSettings();
            $request->save();

            $listing->notifyAboutNewRequest($request);

            if($listing->auto_accept){
              // Get object with all related objects needed to
              // make acceptation process
              $owner = $listing->Owner;
              if ($owner->account_balance <= 0) {
                // change to manual accept
                $listing->auto_accept = false;
                $listing->save();
              } else {
                $request->accept('auto');
              }
            }

            $conn->commit();

            return $request;

        } catch (CreditCardException $e) {
            $conn->rollback();
            return false;
        } catch (Exception $e) {
            $conn->rollback();
            throw $e;
        }
    }

    public function checkCreditCardType($validator, $values)
    {
        $paymentType = isset($values['payment']['balance_card']) && $values['payment']['balance_card'] === 'existing' ? 'existing' : 'new';

        if ($paymentType == 'existing' && isset($this->validatorSchema['payment']['card_id'])) {
            $this->validatorSchema['payment']['card_id']->setOption('required', true);
        } else {
            $this->validatorSchema['payment']['stripeToken']->setOption('required', true);
        }

        return $values;
    }

    public function hasMissingUserDetails()
    {
        return (isset($this['user']['firstname']) || isset($this['user']['lastname']) || isset($this['user']['license']) || isset($this['user']['date_of_birth']) || isset($this['user']['sms_number_verification']) || isset($this['user']['email_number_verification']) || isset($this['user']['mobile_phone']));
    }

    private function mergePaymentForm()
    {
        if ($this->getOption('no_show_fee'))
        {
            $form = new paymentForm(array(), array(
                'user_id' => $this->getOption('user')->getId()
            ));
            $form->configureNoShowFee();
            $form->mergePostValidator(new sfValidatorCallback(array('callback' => array($form, 'checkCreditCard')), array('invalid' => 'Your credit card was not approved. Please check your details again or try another credit card')));
            $this->mergePreValidator(new sfValidatorCallback(array('callback' => array($this, 'checkCreditCardType'))));
            $this->embedForm('payment', $form);
            $this->paymentForm = $form;
        }
    }

    private function mergeUserDetailsForm()
    {
        $form = new  UserDetailsForm(array(), array(
            'user' => $this->getOption('user'),
            'phone_verification'  => false,
            'email_notifications' => true,
            'mobile_phone'        => true,
            'country'             => $this->getOption('listing')->getCountryDomain(),
            'min_age'             => max(18, (int) $this->getOption('listing')->get('min_age'))
        ), false);

        $this->embedForm('user', $form);
        $this->userDetailsForm = $form;
    }

    private function saveUser()
    {
        if ($this->hasMissingUserDetails())
        {
            $this->userDetailsForm->bind($this->getValue('user'));
            $this->userDetailsForm->save();
        }
    }

    private function saveCreditCard($request)
    {
        if ($this->getOption('no_show_fee'))
        {
            $request->credit_card_id     = $this->paymentForm->saveCreditCard($this->getValue('payment'));
            $request->no_show_fee_amount = NO_SHOW_FEE_COST;
        }
        else
        {
            $request->credit_card_id = NULL;
        }
    }

    private function saveUserNotificationSettings()
    {
        $user = $this->getOption('user');
        $user->set('email_newsletter', $this->getValue('email_newsletter'));
        $user->set('email_notify_new_reloc', $this->getValue('email_notify_new_reloc'));

        if (isset($this['sms_notify_update_req']))
        {
            $user->set('sms_notify_update_req', $this->getValue('sms_notify_update_req'));
        }

        $user->save();

        if ($user->get('email_newsletter'))
        {
            //Add user to Mailchimp Web service
            $domain = $user->get('Country')->get('domain');
            $mc = new MailchimpApi();
            $mc->subscribe(sfConfig::get('app_mailchimp_'.$domain.'_list_apikey'), $params['email'], $params['firstname'], $params['lastname']);
        }
    }

    public function renderHiddenFields()
    {
        $output = '';
        foreach ($this->getFormFieldSchema() as $name => $field)
        {
            if ($field->isHidden())
            {
                $output .= $field->render();
            }
        }

        return $output;
    }
}

