<?php
/**
 * Helpers used around the site
 *
 * @package transfercar
 * @subpackage helper
 */


/**
 * Displays link to price table popup
 *
 * Available options:
 *
 *  * icon:  Weather to display an extras icon
 *  * custom_url: Popup will be disabled and url set to custom_url
 *  * other values are passed to link_to attributes
 *
 * @param Doctrine_Record $listing Listing object
 * @param array $options Additional options
 */
function link_to_extras($listing, $options = array())
{
  if (!isset($options['anchor'])) {
    $before = (isset($options['onclick']) ? $options['onclick'] . ';' : '');
    $script = "return GB_showCenter('Optional extras', this.href)";
    $options['onclick'] = "$before $script";
  }
  $name = "Optional extras";

  // weather link should be disabled
  $disabled = !$listing->hasExtras();

  $result = '';
  if (isset($options['icon']) && $options['icon']) {
    $result .= image_tag('optional-extras' . ($disabled ? '-disabled' : '') . '.png') . ' &nbsp;';
    unset($options['icon']);
  }

  if (!$disabled) {
    if (isset($options['anchor'])) {
      $options['href'] = $options['anchor'];
      unset($options['anchor']);
      $result .= content_tag("a", $name, $options);
    } else {
      $result .= link_to($name, "listing/addons?id={$listing->id}", $options);
    }
  } else {
    $result .= '<span class="disabled">' . $name . '</span>';
  }

  return $result;
}

/**
 * Displays link to terms and conditions popup
 *
 * Available options:
 *
 *  * icon:  Weather to display a terms icon
 *  * anchor: Popup will be disabled and url set to anchor
 *  * other values are treated as link attributes
 *
 * @param Doctrine_Record $listing Listing object
 * @param array $options Additional options
 * @returns string
 */
function link_to_terms($listing, $options = array())
{
  if (!isset($options['anchor'])) {
    $before = (isset($options['onclick']) ? $options['onclick'] . ';' : '');
    $script = "return GB_showCenter('Terms and Conditions', this.href)";
    $options['onclick'] = "$before $script";
  }
  $name = "Terms and conditions";

  $result = '';
  if (isset($options['icon']) && $options['icon']) {
    $result .= image_tag('terms-and-conditions.png') . ' &nbsp;';
    unset($options['icon']);
  }

  if (isset($options['anchor'])) {
    $options['href'] = $options['anchor'];
    unset($options['anchor']);
    $result .= content_tag("a", $name, $options);
  } else {
    $result .= link_to($name, "user/terms?id={$listing->terms_id}", $options);
  }

  return $result;
}

/**
 * Shortcut for listing seo optimised link
 *
 * @param string $name Link name
 * @param Doctrine_Record $listing Listing object
 * @param array $options Options for link_to
 * @returns Link tag
 */
function link_to_listing($name, $listing, $options = array())
{ // TODO: remove this helper
  return link_to($name, $listing->getRelocationQuery('listing/view', true), $options);
}

/**
 * display driver requirements
 *
 * @param Listing $listing
 * @param bool $validate Weather to validate currenly logged user
 * @returns string
 */
function driver_requirements($listing, $validate = true)
{
  $user = sfContext::getInstance()->getUser();

  if (!$user->isAuthenticated()) {
    $validate = false;
  }

  $txt = array();
  if($listing->min_age) {
    $txt[] ='Minimum age ' . ($validate && $user->getAge() < $listing->min_age
         ? '<b style="color:#991111">' . $listing->min_age . '</b>'
         : $listing->min_age);
  }

  if($listing->license) {
  	$tmp_listing = $listing->license;
  	if ($tmp_listing == "NZ+AU") $tmp_listing = "AU/NZ";
    $txt[] = 'with '
           . ($validate && $user->license != $listing->license
             ? '<b style="color:#991111">' . $tmp_listing . '</b>'
             : $tmp_listing) . ' license';
  }

  return implode(", ", $txt);
}

function get_ga_module($suffix)
{
  $module = sfContext::getInstance()->getModuleName();
  $action = sfContext::getInstance()->getActionName();

  $routename = sfContext::getInstance()->getRouting()->getCurrentRouteName();
  $prefix = ucfirst($module);

  switch ($routename)
  {
    case 'homepage':
      $prefix = 'Home';
      break;
  }

  switch ($module.'/'.$action)
  {
    case 'request/add':
      $prefix = 'Request';
      break;
  }

  switch ($module) {
    case 'landingpages':
      $pageName = sfContext::getInstance()->getActionStack()->getLastEntry()->getActionInstance()->getVar('currentUrl');
      // throw new Exception(basename($pageName, '.html'));

      $prefix = 'LandingPages_'.ucfirst(basename($pageName, '.html'));
      break;
  }

  return $prefix.','.$suffix;
}

function decorate_widget($page, $block) {
    $content = $page->get($block);

    if (!$content) return;

    $pattern = '#<div class="mceCodeEmbed"><!-- MCE -->(.*)<!-- /MCE --></div>#s';

    if (!preg_match_all($pattern, $content, $matches))
    {
      return $content;
    }

    foreach ($matches as $i => $match)
    {
      $iframe = sprintf('<iframe name="iframe_%s" width="1" height="1" onload="setIframeHeight(this);" style="overflow:hidden" src="%s" frameborder="0" allowfullscreen></iframe>', uniqid(), url_for('landingpages/widget?id='.$page->get('id').'&block='.$block.'&idx='.$i, true));
      $content = str_replace($match[0], $iframe, $content);
    }

    return $content;
}

function _include_gmap($id, $script)
{
  use_helper('Javascript');
  use_javascript('https://maps.googleapis.com/maps/api/js?key='.sfConfig::get('app_gmaps_api_key').'&sensor=false');
  use_javascript('owner/gmaps.js');

  $scriptWrapper = ';(function($) { $(function() { %s }); }(jQuery));';

  $js = javascript_tag(sprintf($scriptWrapper, $script));

  append_slot('javascripts', $js);

  return content_tag('div', '', array('id' => $id, 'class' => 'listing_gmap_route'));
}

function append_slot($name, $value = null)
{
  $slot = get_slot($name);

  if ($value)
  {
    slot($name, $slot.$value);
  }
  else
  {
    slot($name);
    echo $slot;
  }
}

function gmap_widget($lat, $long, $name)
{
  $id = uniqid();
  $name = esc_js($name);

  $script = <<<EOF
$('#$id').gmaps($lat, $long, "$name");
EOF;

  return _include_gmap($id, $script);
}

function gmap_widget_link($from, $to)
{
  $qs = http_build_query(array(
    'origin' => $from->getNameWithCountry(),
    'destination' => $to->getNameWithCountry(),
    'start' => array(
      'lat' => $from->getLatitude(),
      'lon' => $from->getLongitude(),
  )));

  return '/js/map/iframe.php?origin='.$qs;
}

function gmap_widget_route($from, $to, $wrapper = null)
{
    $id = uniqid();
  // $name = esc_js($name);


  $options = json_encode(array(
    'origin' => $from->getNameWithCountry(),
    'destination' => $to->getNameWithCountry(),
    'start' => array(
      'lat' => $from->getLatitude(),
      'lon' => $from->getLongitude(),
    ),
    // 'end' => array(
    //   'lat' => $to->getLatitude(),
    //   'lon' => $to->getLongitude(),
    // ),
  ));
  $script = <<<EOF
$('#$id').gmaps_route($options);
EOF;

  if ($wrapper) {
    $script = sprintf($wrapper, $script);
  }

  return _include_gmap($id, $script);
}

function format_estimated_response_time($estimated_days)
{
  if (!$estimated_days) {
    return __('Not available yet');
  }

  if ($estimated_days > 0 && $estimated_days <= 0.25) {
    return __('Within a few hours');
  }

  return __('Within a day');
}

function format_response_time($responseTimeInDays)
{
  $days = (int) $responseTimeInDays;
  $hours = round(24*($responseTimeInDays - $days));

  return sprintf('%s days, %s hours', $days, $hours);
}

function default_text($text, $default='')
{
  return trim($text) ?: $default;
}

function time_select_tag($name, $listing, $isPickup = true)
{
  $pickup = $listing->get('pickup_time');
  $dropoff = $listing->get('dropoff_time');
  $default = null;
  $min = $max = false;

  if (!$isPickup) {
    $default = $dropoff ? $dropoff : 59400;
    $max = $dropoff ? $default : false;
  } else {
    $default = $pickup ? $pickup : 32400;
    $min = $pickup ? $default : false;
  }

  for ($i = 0; $i < 48; $i++)
  {
    $seconds = ($i * 1800);
    if (($max && $seconds > $max) || ($min && $seconds < $min)) {
      continue;
    }

    $times[$seconds] = format_time($seconds);
  }

  return select_tag($name, options_for_select($times, $default), array('style'=>'width:80px;display:inline-block'));
}

function format_time($time)
{
  $today = strtotime('today');

  return date('h:i A', strtotime('+'.$time.' seconds', $today));
}

function users_select_tag($pager, $default = null)
{
  $query    = clone $pager->getQuery()->getRawValue();

  $dbh = $query->getConnection()->getDbh();

  $sqlQuery = $query
    ->select('Request.id, Request.status, d.id, d.firstname, d.lastname, l.id, o.id, o.company')
    ->offset(0)
    ->limit(0)
    ->getSqlQuery()
    ;

  $stmt = $dbh->prepare($sqlQuery);
  $stmt->execute($query->getParams());

  // Query hydrator is not cloned, we must reset it
  $query->setHydrationMode(null);
  $query->free(true);
  unset($query);

  $drivers = $companies = array();
  while($request = $stmt->fetch())
  {
    $drivers['driver_'.$request['u__id']] = $request['u__firstname'].' '.$request['u__lastname'];
    $companies['company_'.$request['u2__id']] = $request['u2__company'];
  }

  $filters = array('all' => 'Any');

  if (!empty($drivers))
  {
    asort($drivers);
    $filters['drivers'] = $drivers;
  }

  if (!empty($companies))
  {
    asort($companies);
    $filters['companies'] = $companies;
  }

  return select_tag('filters[user]', options_for_select($filters, $default));
}


function url_for_filters($route, $filterForm, $extraQs = array(), $ignore = array())
{
  $request  = sfContext::getInstance()->getRequest();
  $values   = $filterForm->getModifiedValues();

  $urlValues = array();

  foreach ($values as $key => $value)
  {
    $urlValues[$key] = is_array($value) ? implode('|', $value) : $value;
  }

  $extraQs = array_merge(array(
    'sort_col' => $request->getParameter('sort_col', 'extras'),
    'view_by'  => $request->getParameter('view_by', 'list')
  ), $extraQs);

  $params = array_merge($urlValues, $extraQs);

  foreach($ignore as $key => $value) {
    if (isset($params[$key]) && strpos($params[$key], $value) !== false) {
      $params[$key] = str_replace($value, '', $params[$key]);
      $params[$key] = trim($params[$key], '|');
    }
  }

  $qs = http_build_query(array_filter($params));

  return url_for($route).($qs ? '?'.$qs : '');
}

function listing_error_message($listing, $error)
{
    $date = null;
    $message = null;
    $sf_user = sfContext::getInstance()->getUser();

    $message = __($error->getMessage());

    switch ($error->getCode()) {
      case 'expired':
        $date = __('Expired').' '.format_date($listing->get('date_return_by'), 'p');
        break;
      case 'already_requested':
      case 'already_declined':
        $date = __('Requested').' '.str_replace('@', __('at'), format_date($listing->getRequestForUser($sf_user->id)->get('created_at'), 'd MMM yyyy'));
        $message .= '<br/>'.__('View it in').' '.link_to(__('your trips'), '@myTransfercar', array('style' => 'color:#2980b9'));
        break;
      case 'already_booked':
        $date = __('Booked').' '.str_replace('@', __('at'), format_date($listing->getAcceptedRequest()->get('when_accepted'), 'd MMM yyyy @ H:m:s'));
        break;
      case 'closed_in_week_ends':
        $message .= sprintf(
          '<br/><small>'.__('Please request another relocation, or return to request this relocation after %s on Sunday %s time').'</small>',
          format_date($listing->get('Owner')->get('week_end_opening_time'), 'HH:mm'),
          $listing->get('Owner')->get('timezone')
        );
    }

    return array($message, $date);
}

function form_error_class($formField)
{
  return $formField->hasError() ? ' has-error error' : '';
}


function tc_format_date($date, $formatName = 'date')
{
  sfLoader::loadHelpers(array('Date'));
  $sf_user = sfContext::getInstance()->getUser();

  if ($formatName == 'hours' && is_numeric($date))
  {
    $date = strtotime('today') + intval($date);
  }

  try {
    return format_date($date, sfConfig::get('app_'.$formatName.'_time_pattern_'.$sf_user->getUserDomain(), 'p'));
  } catch (Exception $e) {
    return '';
  }
}

function lowest_credit_price($countryId)
{
  $price = Doctrine::getTable('VirtualCurrency')->getLowestPrice($countryId);

  return round($price, 2);
}

function format_license_type($owner, $country)
{
  switch (strtoupper($country->get('domain'))) {
    case 'NZ':
    case 'AU':
      $countryName = __('Australia or New-Zealand');
      break;
    case 'US':
      $countryName = __('the United States or Canada');
      break;
    case 'ZA':
      $countryName = __('South Africa');
      break;
  }
  load_i18n('listing');

  switch ($owner->get('license_type'))
  {
    default:
    case 'full':
      $license = 'A valid and full/unrestricted driving licence issued in %country% only. No international driving licences accepted';
      break;
    case 'full_and_international':
      $license = 'A valid and full/unrestricted driving licence issued in %country%, -or- a valid International Driving Permit, accompanied by a valid driving licence from your country';
      break;
    case 'full_and_translated':
      $license = 'A valid and full/unrestricted driving licence issued in %country%, -or- a legally confirmed English translation of your driving licence, accompanied by a valid driving licence from your country';
      break;
    case 'full_or_foreign':
      $license = 'A valid and full/unrestricted driving licence issued in %country%, -or- a valid driving licence from your country';
      break;
  }

  return __($license, ['%country%' => $countryName]);
}


function get_nb_listings()
{
  static $nb = null;

  if ($nb === null)
  {
    $country = sfContext::getInstance()->getUser()->getCountry();
    $query = Doctrine_Query::create()
            ->select('SUM(l.nb_listings) nb')
            ->from('Listing l')
            ->innerJoin('l.PickupLocation lp')
            ->innerJoin('lp.Category lpC')
            ->where('l.status = ?', Listing::STATUS_NEW)
            ->addWhere("l.date_return_by >= ?", date('Y-m-d'))
            ->addWhere("lpC.country_id = ?", $country);

    $data = $query->execute([], Doctrine::HYDRATE_NONE);
    $nb1 = (int) $data[0][0];

    $query = Doctrine_Query::create()
            ->select('count(*) nb')
            ->from('GhostListing l')
            ->innerJoin('l.PickupLocation lp')
            ->innerJoin('lp.Category lpC')
            ->where("lpC.country_id = ?", $country);

    $data = $query->execute([], Doctrine::HYDRATE_NONE);
    $nb2 = (int) $data[0][0];

    $nb = $nb1 + $nb2;
  }

  return $nb;
}


function include_hreflang_tags()
{
    $country = sfContext::getInstance()->getUser()->getUserDomain();
    $request = sfContext::getInstance()->getRequest();

    $links = [];
    $links[] = sprintf('<link rel="canonical" href="https://%s"/>', sfConfig::get('app_domain_'.$country));
    $links[] = sprintf('<meta property="og:url" content="https://%s/"/>', sfConfig::get('app_domain_'.$country));

    $countries = LocationCountry::getAll();

    foreach ($countries as $row)
    {
      if ($row['domain'] != $country)
      {
        $links[]= sprintf('<link rel="alternate" hreflang="en_%s" href="https://%s/"/>', strtoupper($row['domain']), sfConfig::get('app_domain_'.$row['domain']));
      }
    }

    // echo implode("\n", $links);
}


function tc_thumbnail($image, $style, $attributes = array())
{
  use_helper('Thumbnail');

  $config = sfConfig::get('app_thumbnail_styles');

  if (!isset($config[$style])) {
    return image_tag($image);
  }

  $config = array_merge(array(
    'options' => [],
    'mode'    => 'stretch',
  ), $config[$style]);

  $config['options'] = array_merge($config['options'], $attributes);

  //$config['options']['width'] = $config['width'];
  //$config['options']['height'] = $config['height'];

  return image_tag('/uploads/thumbs/'.$style.$image, $config['options']);
}

function tc_thumbnail_path($image, $style, $attributes = array())
{
  $config = sfConfig::get('app_thumbnail_styles');

  if (!isset($config[$style])) {
    return $image;
  }

  return '/uploads/thumbs/'.$style.$image;
}

function push_ga_pageview($url)
{
  $script = <<<EOF
;window.dataLayer && dataLayer.push({
"event":"VirtualPageview",
"virtualPageURL":"%s",
"virtualPageTitle" : "%s"
});
EOF;

  return sprintf($script, $url, str_replace(array('/', '_'), array('', ' '), $url));;
}

function url_for_request($route_name, $parameters = array())
{
  $context = sfContext::getInstance();
  $url = $context->getRequest()->getRelativeUrlRoot();

  if ($parameters instanceof sfOutputEscaper)
  {
    $parameters = $parameters->getRawValue();
  }

  if (!sfConfig::get('sf_no_script_name'))
  {
    $scriptName = $context->getRequest()->getScriptName();
    $url = is_null($url) ? $scriptName : $url.'/'.basename($scriptName);
  }

  $url .= $context->getRouting()->generate($route_name, $parameters);

  return $url;
}

function tc_search_stats_beacon($filter_form, $pager, $nb_alternatives = 0)
{
  $filters = $filter_form->getModifiedValues();

  if ($pager->getPage() != 1 || !(isset($filters['pickup']) || isset($filters['dropoff']))) {
    return;
  }

  $params = array(
    'nb_listings='.$pager->getNumRelocations(),
    'nb_sponsored_ads='.$nb_alternatives
  );

  foreach (array('pickup', 'dropoff', 'from_date') as $key) {
    if (isset($filters[$key]) && $filters[$key]) {
      $keyName = 'from_date' == $key ? 'pickup_date' : $key;
      $params[] = $keyName.'='.$filters[$key];
    }
  }

  return image_tag(url_for('@search_stats_beacon?'.implode('&', $params)), ['alt' => '']);
}

function tc_help_link_register()
{
  $links = array(
    'nz' => 'https://www.transfercar.co.nz/help/customer/portal/articles/1570925-how-do-i-register-as-a-driver-in-nz-?b_id=2761',
    'au' => 'https://www.transfercar.com.au/help/customer/portal/articles/1570935-how-to-register-as-a-driver-in-australia-?b_id=2760',
    'us' => 'https://www.transfercarus.com/help/customer/portal/articles/1570929-how-do-i-register-as-a-transfercar-driver-in-the-us-?b_id=2763'
  );

  $domain = sfContext::getInstance()->getUser()->getUserDomain();

  return $links[$domain];
}

function no_show_fee_amount()
{
  if (!defined('NO_SHOW_FEE')) {
      define('NO_SHOW_FEE', Settings::getNoShowFeeStatus());
      define('NO_SHOW_FEE_COST', Settings::getNoShowFeeCost());
  }

  return NO_SHOW_FEE ? NO_SHOW_FEE_COST : 0;
}
