<?php

class tcListingRequestValidator extends sfValidatorBase
{
  private $driver_requirements;

  protected function configure($options = array(), $messages = array())
  {
    $this->addRequiredOption('listing');
    $this->addRequiredOption('user');

    $this->addMessage('covid_19', 'Due to the Covid-19 crisis, all non-essential travel relocations is limited');

    $this->addMessage('deleted', 'The rental car company has removed this relocation');
    $this->addMessage('cancelled', 'The rental car company has cancelled this relocation');
    $this->addMessage('expired', 'The relocation was not booked and timed out');
    $this->addMessage('already_requested', 'You have already requested this relocation');
    $this->addMessage('already_booked', 'The rental car company has booked this relocation');
    $this->addMessage('already_declined', 'The rental car company declined your request');
    $this->addMessage('driver_requirements', 'You do not fulfill the following requirements:');
    $this->addMessage('too_many_requests', 'You can\'t have more than three pending requests');
    $this->addMessage('closed_in_week_ends', 'This rental car company does not accept requests during weekends');
    $this->addMessage('ro_ban', 'You can\'t request vehicles from this rental operator');
    $this->addMessage('tc_ban', 'Your account has been blocked, please contact our support team for more details');
    $this->addMessage('suspect', 'You do not fulfill the requirements, Please contact our support team for more details');
    $this->addMessage('alternative_dropoff', 'Incorrect dropoff location');
  }

  public function hasError(array $values = array())
  {
    try
    {
      $this->doClean($values);
    }
    catch (sfValidatorError $error)
    {
      return $error;
    }

    return false;
  }

  public function doClean($values)
  {
    if ($this->getOption('user')->isAuthenticated())
    {
        $this->doCleanUser($values);
    }

    $this->doCleanListing($values);

    return $values;
  }

  private function doCleanListing($values)
  {
    $listing = $this->getOption('listing');
    $dropoff = isset($values['dropoff']) && $values['dropoff'] ? Doctrine::getTable('OwnerBranch')->find($values['dropoff']) : false;
    $message = in_array($this->getOption('user')->getUserDomain(), ['xx']) ? 'covid_19': false;

    if ($listing->get('Owner')->get('closed_in_week_end') && $this->areReservationClosed($listing))
    {
      $message = 'closed_in_week_ends';
    }
    elseif ($listing->status == Listing::STATUS_DELETED)
    {
      $message = 'deleted';
    }
    elseif ($listing->status == Listing::STATUS_CANCELED)
    {
      $message = 'cancelled';
    }
    elseif ($listing->status == Listing::STATUS_EXPIRED)
    {
      $message = 'expired';
    }
    elseif($listing->get('accepted_request_id'))
    {
      $message = 'already_booked';
    } 
    elseif ($dropoff && $dropoff->get('owner_id') != $listing->get('owner_id')) 
    {
      $message = 'alternative_dropoff';
    } 
    elseif (isset($values['dropoff']) && $values['dropoff'] && !$dropoff) 
    {
      $d2 = Doctrine::getTable('OwnerBranch')->find($values['dropoff']);
      $message = 'alternative_dropoff';
    }

    $this->throwError($message);
  }

  private function doCleanUser($values)
  {
    $user = $this->getOption('user');
    $listing = $this->getOption('listing');
    $profile = $user->getAttribute('profile');
    $profile->refresh();
    $message = false;

    if (false && $profile && $profile->get('user_status') == 'blocked')
    {
      $message = 'tc_ban';
    }
    elseif (false && $profile && $profile->isBannedFor($listing->get('owner_id')))
    {
      $message = 'ro_ban';
    }
    elseif (false && $profile && Doctrine::getTable('User')->isSuspect($profile))
    {
      $message = 'suspect';
    }
    elseif ($listing->get('accepted_request_id') && $listing->isBookedBy($user->id))
    {
      $message = 'already_requested';
    }
    elseif ($req = $listing->getRequestForUser($user->id))
    {
      switch ($req->get('status')) {
        case Request::STATUS_PENDING:
          $message = 'already_requested';
          break;
        case Request::STATUS_REJECTED:
          $message = 'already_declined';
          break;
        case Request::STATUS_OWNER_CANCELED:
          $message = 'cancelled';
          break;
        case Request::STATUS_CANCELED:
          $message = 'already_declined';
          break;
        case Request::STATUS_DRIVER_CANCELED:
        // case Request::STATUS_CANCELED_BY_DRIVER:
          $message = 'already_requested';
          break;

      }
    }
    elseif ($user->getAttribute('profile')->getPendingRequests()->count() >= sfConfig::get('app_driver_max_pending_requests', 3))
    {
      $message = 'too_many_requests';
    }
    elseif (($req = $user->getListingRequirements($listing)) && !empty($req))
    {
      $message = 'driver_requirements';
      $this->driver_requirements = $req;
    }
    elseif (isset($values['pickup_date']))
    {
      $validator = new tcRequestedDateValidator(array(
        'listing' => $this->getOption('listing')
      ));

      $validator->clean($values);
    }

    $this->throwError($message);
  }

  public function getDriverRequirements()
  {
    return $this->driver_requirements;
  }

  private function areReservationClosed(Listing $listing)
  {
    return $listing->areReservationClosed();
  }

  private function throwError($message)
  {
    if ($message !== false)
    {
        throw new sfValidatorError($this, $message);
    }
  }
}