<?php

class tcRequestedDateValidator extends sfValidatorBase
{
    private $request;

    public function configure($options = array(), $messages = array())
    {
        $this->addRequiredOption('listing');
        $this->addMessage('invalid', 'Requested dates are invalid');
        $this->addMessage('non_consecutive', 'Requested dates must be consecutives');
        $this->addMessage('overlapping_request', 'This request overlaps with another accepted request you made.');
    }

    protected function doClean($values)
    {
        $listing = $this->getOption('listing');

        $listing_pickup_ts = strtotime($listing->date_available_from);
        $listing_return_ts = strtotime($listing->date_return_by);

        $date = $values['pickup_date'];
        $request_return_ts = strtotime($values['dropoff_date']);

        while (strtotime($date) <= $request_return_ts)
        {
            $requested_ts = strtotime($date);
            $requests = $listing->getConflictingRequests($requested_ts);

            if (false !== $requests)
            {
                foreach ($requests as $request)
                {
                    if ($request->overlaps($listing, $date, $date))
                    {
                        $this->request = $request;

                        throw new sfValidatorError($this, 'overlapping_request', array(
                            'date' => date('Y-m-d', $requested_ts)
                        ));
                    }
                }
            }

            if ($requested_ts === false || $requested_ts < $listing_pickup_ts || $requested_ts > $listing_return_ts)
            {
                throw new sfValidatorError($this, 'invalid', array('value' => $date));
            }

            $date = date('Y-m-d', strtotime('+1 day', $requested_ts));
        }

        $values = $this->getDays($listing, $values);

        // post validate calculated dates and days
        return $this->validateDays(array_merge($values, array(
            'dropoff_date' => $values['dropoff_date'],
            'pickup_date' => $values['pickup_date']
        )));
    }
    private function format_time($t,$f=':') // t = seconds, f = separator 
    {  
        return sprintf("%02d%s%02d%s%02d", floor($t/3600), $f, ($t/60)%60, $f, $t%60);
    } 

    private function getDays($listing, $values)
    {
        $date = strtotime($values['pickup_date'].' '.$this->format_time($values['pickup_time']));
        $end_ts = strtotime($values['dropoff_date'].' '.$this->format_time($values['dropoff_time']));

        list($free_days, $paid_days) = $listing->getDaysFor($date, $end_ts);

        $values['free_days'] = $free_days;
        $values['paid_days'] = $paid_days;
      
        return $values;
    }

    /**
     * If selected dates are invalid, the outcome will be wrong free/paid days
     *
     * For instance, if 4 days are available, but a date range of 6 days
     * is given, this will produce 4 free days + 2 paid days, which is more
     * than allowed.
     */
    private function validateDays($values)
    {
        $listing = $this->getOption('listing');
        $error = false;

        if ($values['paid_days'] > $listing->get('paid_days_allowed')) {
            $error = true;
        }


        if ($values['free_days'] > $listing->get('days_allowed')) {
            $error = true;
        }

        if ($error) {
            throw new sfValidatorError($this, 'invalid');
        }

        return $values;
    }

    private function getDistance($dropoff, $pickup)
    {
        $lat1 = floatval($dropoff->get('lat'));
        $lat2 = floatval($pickup->get('lat'));

        $lon1 = floatval($dropoff->get('lon'));
        $lon2 = floatval($pickup->get('lon'));

        return sqrt(pow(110.56*($lat2 - $lat1), 2) + pow(84.8*($lon2 - $lon1), 2));
    }

    public function getRequest()
    {
        return $this->request;
    }
}
