<?php

class TcStats
{
    const DAILY_QUERY = 'daily';
    const WEEKLY_QUERY = 'weekly';
    const MONTHLY_QUERY = 'monthly';
    const YEARLY_QUERY = 'yearly';

    private $conn;

    public function __construct($conn)
    {
        $this->conn = $conn;
    }

    public function findAll($searchType, $from, $to, $country = null, $user = null)
    {
        $from = $this->getDate($from);
        $to   = $this->getDate($to);

        $query = $this->conn->prepare($sql = $this->getAllQueryFor($searchType, (bool) $country, (bool) $user));

        $params = array($from, $to);

        if ($country) {
          array_unshift($params, $country);
        }

        if ($user) {
          array_unshift($params, $user);
        }

        $query->execute($params);

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    private function getAllQueryFor($searchType, $withCountry, $withUser)
    {
        $query = false;
        switch ($searchType) {
            case self::DAILY_QUERY:
                $query = $this->getAllDailyQuery();
                break;

            case self::WEEKLY_QUERY:
                $query = $this->getAllWeeklyQuery();
                break;

            case self::MONTHLY_QUERY:
                $query = $this->getAllMonthlyQuery();
                break;

            case self::YEARLY_QUERY:
                $query = $this->getAllYearlyQuery();
                break;

            default:
                throw new Exception("Unknown search type");
        }

        $query = $this->addCountry($query, $withCountry);
        $query = $this->addUser($query, $withUser);

        return $query;
    }

    private function addCountry($query, $withCountry)
    {
      if ($withCountry) {
        $query = str_replace('WHERE', 'WHERE country_id = ? AND', $query);
      }

      return $query;
    }

    private function addUser($query, $withUser)
    {
      if ($withUser) {
        $query = str_replace('WHERE', 'WHERE user_id = ? AND', $query);
      }

      return $query;
    }

    private function getAllDailyQuery()
    {
      return 'SELECT year, month, day,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_listings) as nb_listings,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected, sum(nb_new_requests) as nb_new_requests
                  FROM tc_listing_stats
                  WHERE STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') >= ?
                  AND STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') <= ?
                  GROUP BY day, month, year
                  ORDER BY year, month, day
                ';
    }

    private function getAllWeeklyQuery()
    {
      return 'SELECT year, month, week,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_listings) as nb_listings,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected, sum(nb_new_requests) as nb_new_requests
                  FROM tc_listing_stats
                  WHERE STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') >= ?
                  AND STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') <= ?
                  GROUP BY week, year
                  ORDER BY year, month, week
                ';
    }

    private function getAllMonthlyQuery()
    {
      return 'SELECT year, month,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_listings) as nb_listings,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected, sum(nb_new_requests) as nb_new_requests
                  FROM tc_listing_stats
                  WHERE STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') >= ?
                  AND STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') <= ?
                  GROUP BY month, year
                  ORDER BY year, month
                ';
    }
    public function findTop5($searchType, $from, $to, $country = null)
    {
        $from = is_numeric($from) ? date('Y-m-d', $from) : $from;
        $to   = is_numeric($to) ? date('Y-m-d', $to) : $to;

        $query = 'SELECT user_id, username, company,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_new_requests) as nb_new_requests,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected,
                        sum(nb_new_listings + nb_new_requests + nb_relos + nb_no_show + nb_rejected) as total
                  FROM tc_listing_stats
                  WHERE STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') >= ?
                  AND STR_TO_DATE(concat(year, "-", month, "-", day), \'%Y-%m-%d\') <= ?
                  GROUP BY user_id, username, company
                  ORDER BY total DESC, user_id, username
                  LIMIT 5';

        $query = $this->conn->prepare($q = $this->addCountry($query, (bool) $country));

        $params = array($from, $to);

        if ($country) {
          array_unshift($params, $country);
        }

        $query->execute($params);

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    public function find($searchType, $id, $fetchMode = PDO::FETCH_OBJ)
    {
        $query = $this->conn->prepare($this->getQueryFor($searchType));
        $query->execute(array($id));

        return $query->fetchAll($fetchMode);
    }

    private function getQueryFor($searchType)
    {
        switch ($searchType) {
            case self::DAILY_QUERY:
                return $this->getDailyQuery();

            case self::WEEKLY_QUERY:
                return $this->getWeeklyQuery();

            case self::MONTHLY_QUERY:
                return $this->getMonthlyQuery();

            case self::YEARLY_QUERY:
                return $this->getYearlyQuery();

            default:
                throw new Exception("Unknown search type");
        }
    }

    private function getDailyQuery()
    {
        return 'SELECT user_id, username, company, year, month, day,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_new_requests) as nb_new_requests,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected
                  FROM tc_listing_stats
                  WHERE user_id = ?
                  GROUP BY user_id, username, company, day, month, year
                  ORDER BY user_id, username, company, year, month, day
                ';
    }

    private function getWeeklyQuery()
    {
        return 'SELECT user_id, username, company, year, month, week,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_new_requests) as nb_new_requests,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected
                  FROM tc_listing_stats
                  WHERE user_id = ?
                  GROUP BY user_id, username, company, week, month, year
                  ORDER BY user_id, username, company, year, month, week
                ';
    }

    private function getMonthlyQuery()
    {
        return 'SELECT user_id, username, company, month, year,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_new_requests) as nb_new_requests,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected
                  FROM tc_listing_stats
                  WHERE user_id = ?
                  GROUP BY user_id, username, company, month, year
                  ORDER BY user_id, username, company, year, month
                ';
    }

    private function getYearlyQuery()
    {
        return 'SELECT user_id, username, company, year,
                        sum(nb_new_listings) as nb_new_listings, sum(nb_new_requests) as nb_new_requests,
                        sum(nb_relos) as nb_relos, sum(nb_no_show) as nb_no_show,
                        sum(nb_rejected) as nb_rejected
                  FROM tc_listing_stats
                  WHERE user_id = ?
                  GROUP BY user_id, username, company, year
                  ORDER BY user_id, username, company, year
                ';
    }

    private function getDate($date)
    {
      if (!is_numeric($date)) {
        $date = strtotime($date);
      }

      return date('Y-m-d', $date);
    }
}