<?php

class UpdateListingStats
{
    private $conn;

    public function __construct($conn)
    {
        $this->conn = $conn;
    }

    public function update($lastUpdateTs)
    {
        $today = strtotime('today');

        $lastUpdateTs = is_numeric($lastUpdateTs) ? $lastUpdateTs : strtotime($lastUpdateTs);

        $nbRowsInserted = 0;

        while ($lastUpdateTs <= $today)
        {
            $stats = array();
            $date = date('Y-m-d', $lastUpdateTs);

            $this->computeStats($stats, $this->getListings($date), 'nb_listings');
            $this->computeStats($stats, $this->getNewListings($date), 'nb_new_listings');
            $this->computeStats($stats, $this->getListedListings($date), 'nb_listed_listings');
            $this->computeStats($stats, $this->getRequests($date), 'nb_requests');
            $this->computeStats($stats, $this->getNewRequests($date), 'nb_new_requests');
            $this->computeStats($stats, $this->getRelos($date), 'nb_relos');
            $this->computeStats($stats, $this->getnoShow($date), 'nb_no_show');
            $this->computeStats($stats, $this->getRejected($date), 'nb_rejected');

            foreach ($stats as $owner_id => $stat) {
                $nbRowsInserted += $this->updateStat($owner_id, $stat, $lastUpdateTs);
            }

            $lastUpdateTs = strtotime('+1 day', $lastUpdateTs);
        }

        $query = $this->conn->prepare('UPDATE user u set total_relocations = (SELECT SUM(nb_relos) from tc_listing_stats s where s.user_id=u.id)');
        $query->execute();

        return array('name' => 'listing', 'inserted' => $nbRowsInserted);
    }

    public function computeStats(&$stats, $values, $type)
    {
        foreach ($values as $stat)
        {
            $this->addToStats($stats, $stat->owner_id.'-'.$stat->country_id, $type, $stat->count);
            $stats[$stat->owner_id.'-'.$stat->country_id]['username'] = $stat->username;
            $stats[$stat->owner_id.'-'.$stat->country_id]['company'] = $stat->company;
            $stats[$stat->owner_id.'-'.$stat->country_id]['country_id'] = $stat->country_id;
            $stats[$stat->owner_id.'-'.$stat->country_id]['owner_id'] = $stat->owner_id;
        }
    }

    public function addToStats(&$stats, $owner_id, $type, $nb)
    {
        if (!isset($stats[$owner_id])) {
            $stats[$owner_id] = array();
        }

        if (!isset($stats[$owner_id][$type])) {
            $stats[$owner_id][$type] = 0;
        }

        $stats[$owner_id][$type] += $nb;
    }

    public function updateStat($owner_id, $stat, $date)
    {
        $insertQuery = 'REPLACE into tc_listing_stats
                (user_id, country_id, day, week, month, year, username, company, nb_listed_listings, nb_listings,
                 nb_new_listings, nb_new_requests, nb_requests, nb_relos, nb_no_show, nb_rejected)
                VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
                ';

        $query = $this->conn->prepare($insertQuery);
        $query->execute(array(
            $stat['owner_id'],
            $stat['country_id'], // country_id
            date('j', $date),
            date('W', $date),
            date('n', $date),
            date('Y', $date),
            $stat['username'], // username
            $stat['company'], // company
            isset($stat['nb_listed_listings']) ? $stat['nb_listed_listings'] : 0,
            isset($stat['nb_listings']) ? $stat['nb_listings'] : 0,
            isset($stat['nb_new_listings']) ? $stat['nb_new_listings'] : 0,
            isset($stat['nb_new_requests']) ? $stat['nb_new_requests'] : 0,
            isset($stat['nb_requests']) ? $stat['nb_requests'] : 0,
            isset($stat['nb_relos']) ? $stat['nb_relos'] : 0,
            isset($stat['nb_no_show']) ? $stat['nb_no_show'] : 0,
            isset($stat['nb_rejected']) ? $stat['nb_rejected'] : 0,
        ));
        return $query->rowCount();
    }

    public function getLastUpdateTimestamp()
    {
        $result = $this->conn->execute('select max(created_at) as max from tc_listing_stats');

        $ts = $result->fetch(PDO::FETCH_OBJ)->max;

        return $ts ? $ts : 0;
    }

    protected function getListings($ts)
    {
        if ($ts === date('Y-m-d')) {
            $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                           from listing l
                                           inner join locations lp on l.pickup_location_id=lp.id
                                           inner join location_category lc on lp.category_id=lc.id
                                           inner join user u on u.id=l.owner_id
                                           where l.status = 0
                                           group by owner_id, username, country_id, company');

            $query->execute(array());
            return $query->fetchAll(PDO::FETCH_OBJ);
        }

        return array();
    }

    protected function getListedListings($ts)
    {
        if ($ts === date('Y-m-d')) {
            $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, sum(nb_listings) as count
                                           from listing l
                                           inner join locations lp on l.pickup_location_id=lp.id
                                           inner join location_category lc on lp.category_id=lc.id
                                           inner join user u on u.id=l.owner_id
                                           where l.status = 0
                                           group by owner_id, username, country_id, company');

            $query->execute(array());
            return $query->fetchAll(PDO::FETCH_OBJ);
        }

        return array();
    }

    protected function getNewListings($ts)
    {
        $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                       from listing l
                                       inner join locations lp on l.pickup_location_id=lp.id
                                       inner join location_category lc on lp.category_id=lc.id
                                       inner join user u on u.id=l.owner_id
                                       where l.created_at = ?
                                       group by owner_id, username, country_id, company');

        $query->execute(array($ts));
        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    protected function getNewRequests($ts)
    {
        $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                       from listing l inner join request r on r.listing_id=l.id
                                       inner join locations lp on l.pickup_location_id=lp.id
                                       inner join location_category lc on lp.category_id=lc.id
                                       inner join user u on u.id=l.owner_id
                                       where DATE_FORMAT(r.created_at, \'%Y-%m-%d\') = ?
                                       group by owner_id, username, country_id, company');

        $query->execute(array($ts));

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    protected function getRequests($ts)
    {
        if ($ts === date('Y-m-d')) {

            $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                           from listing l inner join request r on r.listing_id=l.id
                                           inner join locations lp on l.pickup_location_id=lp.id
                                           inner join location_category lc on lp.category_id=lc.id
                                           inner join user u on u.id=l.owner_id
                                           where r.status = 0
                                           group by owner_id, username, country_id, company');

            $query->execute(array());

            return $query->fetchAll(PDO::FETCH_OBJ);
        }

        return array();
    }

    protected function getRelos($ts)
    {
        $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                       from listing l
                                       inner join locations lp on l.pickup_location_id=lp.id
                                       inner join location_category lc on lp.category_id=lc.id
                                       inner join request r on r.listing_id=l.id
                                       inner join user u on u.id=l.owner_id
                                       where DATE_FORMAT(r.when_accepted, \'%Y-%m-%d\') = ?
                                       AND r.status in (1, 4, 8, 9, 10)
                                       group by owner_id, username, country_id, company');

        $query->execute(array($ts));

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    protected function getNoShow($ts)
    {
        $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                       from listing l inner join request r on r.listing_id=l.id
                                       inner join locations lp on l.pickup_location_id=lp.id
                                       inner join location_category lc on lp.category_id=lc.id
                                       inner join user u on u.id=l.owner_id
                                       where DATE_FORMAT(r.created_at, \'%Y-%m-%d\') = ?
                                       AND (r.no_show_fee in (1 , 2) OR r.status in (8, 9, 10))
                                       group by owner_id, username, country_id, company');

        $query->execute(array($ts));

        return $query->fetchAll(PDO::FETCH_OBJ);
    }

    protected function getRejected($ts)
    {
        $query = $this->conn->prepare('SELECT owner_id, username, country_id, company, count(*) as count
                                       from listing l inner join request r on r.listing_id=l.id
                                       inner join locations lp on l.pickup_location_id=lp.id
                                       inner join location_category lc on lp.category_id=lc.id
                                       inner join user u on u.id=l.owner_id
                                       where DATE_FORMAT(r.created_at, \'%Y-%m-%d\') = ?
                                       AND r.status = 2
                                       group by owner_id, username, country_id');

        $query->execute(array($ts));

        return $query->fetchAll(PDO::FETCH_OBJ);
    }
}