<?php

namespace Transfercar\Email\Sender;

// symfony classes
use sfLoader;
use sfConfig;


// doctrine classes
use Doctrine;
use Doctrine_Query;
use Doctrine_Pager;

// TC classes
use Listing;
use LocationCountry;
use Transfercar\Mandrill\TcMandrill;

class NewListingNotificationEmail
{
    private $logger;

    public function __construct($logger)
    {
        $this->logger = $logger;
        sfLoader::loadHelpers(array("Helper","Tag", "Url", "Transfercar", "I18N"));
    }

    protected function log($message)
    {
        $this->logger->log(sprintf('{%s} %s', __CLASS__, $message));
    }

    public function sendInstantEmails($country)
    {
        $userIds = $this->sendEmails('instance', $country);

        $this->deleteAlerts($userIds);
    }

    public function sendDailyEmails($country)
    {
        $userIds = $this->sendEmails('daily', $country);

        $this->deleteAlerts($userIds);
    }

    private function deleteAlerts($userIds)
    {
        if (count($userIds)) {
            $query = Doctrine_Query::create()
                ->delete()
                ->from('Alert a')
                ->whereIn('user_id', $userIds)
                ->execute();
        }
    }

    protected function getUsers($freq, $country_id)
    {
        $freq = str_replace('instance', 'instant', $freq);
        $query = Doctrine_Query::create()
            ->select('nu.*')
            ->from('NotificationUser nu')
            ->innerJoin('nu.Alerts a')
            ->where('nu.frequence = ?', $freq)
            ->addWhere('nu.country_id = ?', $country_id);

        // $users = $query->execute();

        // $query->free(true);

        $pager = new Doctrine_Pager($query, 1, 100);

        return $pager;
    }

    protected function getMatchingListingsForUser($user, $freq, $country_id)
    {
        $listings = array();
        $ids = [-1]; // -1 by default otherwise if the driver do not have any alert set, all the new listings will match

        foreach ($user->get('Alerts') as $alert) {
            $ids[] = $alert->get('source_id');
        }

        $query =  Doctrine::getTable('Listing')
            ->createQuery('l')
            ->whereIn('l.id', $ids)
            ->addWhere('l.status = ?', Listing::STATUS_NEW);

        $listings = $query->execute();
        $query->free(true);

        return $listings;
    }

    protected function sendEmails($freq, $country_domain, $useNew = false)
    {
        $country_id = LocationCountry::retreiveByDomain($country_domain);

        $userIds = array();

        $sent = 0;

        $this->log('Start email for domain '.$country_domain);

        $country_domain = $country_domain;
        $site = sfConfig::get('app_domain_' . $country_domain);

        // Get Users with routes or with the notify new listing flag
        $pager = $this->getUsers($freq, $country_id);
        $batch =  $receivers = [];

        do {
            $users = $pager->execute();
            $lastpage = $pager->getLastPage();
            $userIds = array_merge($userIds, $this->sendNewListingNotification($users, $freq, $country_id, $site));
            $pager->setPage($pager->getPage()+1);
        } while ($pager->getPage() <= $lastpage);

        $this->log(sprintf('Found %s users', count(array_unique($userIds))));

        return array_unique($userIds);
    }


    public function getRelocationPath($listing){
        $pattern = '/[^a-zA-Z0-9]/';
        $from = preg_replace($pattern, '_', $listing->PickupLocation->name);
        $to = preg_replace($pattern, '_', $listing->DropoffLocation->name);
        //$path = 'relocation/from='.$from.'&to='.$to.'&relo_id='. $listing->id;
        $path = 'relocation/'.$from.'/'.$to.'/'. $listing->id.".html";
        return $path;
    }

    private function sendNewListingNotification($users, $freq, $country_id, $site)
    {
        $batch =  $receivers = $userIds = [];
        foreach ($users as $user)
        {
            // ignore unsubscribed users
            if ($user->get('user_id') && !$user->get('User')->get('email_notify_new_reloc')) 
            {
                $this->deleteAlerts([$user->getId()]);
                continue;
            }
            // get listings matching with the user routes
            $listings = $this->getMatchingListingsForUser($user, $freq, $country_id);
            $this->log('nb listings: '.count($listings));
            $userIds[] = $user->getId();

            if (count($listings) == 0)
            {
                $this->deleteAlerts([$user->getId()]);
                $user->free(true);
                unset($user, $listings);
                continue;
            }

            $pickups = [];
            foreach ($listings as $listing)
            {
                if (!isset($pickups[$listing->get('PickupLocation')->get('name')]))
                {
                    $pickups[$listing->get('PickupLocation')->get('name')] = array();
                }

                $pickups[$listing->get('PickupLocation')->get('name')][] = $listing;
            }

            $email_data = [];
            foreach ($pickups as $name => $listings) {
                $data = array('pickup' => $name, 'dropoffs' => array());

                // we show only 3 listings per route
                foreach (array_slice($listings, 0, 3) as $listing) {
                    try {
                        $price = (int) $listing->getIncludedDaysPrice();
                    } catch (\Exception $e) {
                        $price = 0;
                    }

                    if ($price)
                    {
                      $days = format_number_choice('[1] 1 day |(1,+Inf] %days% days', array('%days%' => $listing->getDaysAllowedAvailable()), $listing->getDaysAllowedAvailable()).' at $'.$price;
                    }
                    else
                    {
                      $days = format_number_choice('[1] 1 free day |(1,+Inf] %days% free days', array('%days%' => $listing->getDaysAllowedAvailable()), $listing->getDaysAllowedAvailable());
                    }

                    $data['dropoffs'][] = array(
                        'route_title'  => $listing->getTitle(),
                        'image'        => 'https://' . $site . '/'.$listing->getVehicle()->icon_path(),
                        'link'         => 'https://' . $site . '/'.$this->getRelocationPath($listing),
                        'nb_days'      => $days,
                        'from_date'    => tc_format_date($listing->get('date_available_from'), 'short'),
                        'to_date'      => tc_format_date($listing->get('date_return_by'), 'short'),
                        'vehicle_type' => $listing->get('VehicleType')->getName()
                    );
                }

                $email_data[] = $data;
            }

            if(count($email_data) > 0)
            {
                $receivers[] = $user->getEmailToParams();
                $batch[] = array(
                    'rcpt' => $user->getUserEmail(),
                    'vars' => array('routes' => $email_data)
                );
            }

            $user->free(true);
            unset($user, $listings);

            if (count($batch) == 10) {
                $this->sendNotification($receivers, $batch);
                $this->deleteAlerts($userIds);
                $batch = $receivers = $userIds = [];
            }
        }

        if (count($batch))
        {
            $this->sendNotification($receivers, $batch);
        }

        return $userIds;
    }

    private function sendNotification($receivers, $batch, $retry = 0)
    {
        $this->log('send batch');
        $mandrill = new TcMandrill();

        try {
            $mandrill->sendTemplate('New relocations available in {{COUNTRY}}', $receivers, array('merge_vars' => $batch), 'new-listing-notification');
        } catch (\Mandrill_Error $e) {
            if (strpos($e->getMessage(), '504 Gateway Time-out') !== false && $retry < 3) {
                $this->log('Gateway time-out, wait');
                sleep(60);
                $this->sendNotification($receivers, $batch, $retry + 1);
            }
        }
    }
}