<?php

namespace Transfercar\Email\Sender;

use Transfercar\Mandrill\TcMandrill;
use Transfercar\ElasticSearch\SuggestedListings;

use Doctrine;
use Listing;
use Request;
use sfLoader;
use sfConfig;
use User;

class RequestNotificationEmail extends RelocationEmail
{
    private
        $request,
        $listing,
        $domain,
        $groupedRequests;

    /**
     * RequestNotificationEmail objects are used to build and send emails to RO and drivers
     *
     * @param Request $request A Transfercar Request object instance
     * @param array $groupedRequests Array of requests to display in the email
     */
    public function __construct(Request $request, $groupedRequests = array())
    {
        $this->request         = $request;
        $this->listing         = $request->get('Listing');
        $this->domain          = $this->listing->getCountryDomain();
        $this->groupedRequests = $groupedRequests;
    }

    /**
     * Send the notification email to the recipient
     *
     * @param string $type The email template name without the domain suffix
     * @param mixed $to Instance of User or OwnerUser
     */
    public function send($type, $to)
    {
        $email_data = $this->build($type, $to);

        $mandrill = new TcMandrill($this->request->getCountryDomain());
        $mandrill->sendTemplate(
            $this->getSubject($type),
            self::getRecipient($to),
            $email_data,
            $this->fixType($type)
        );
    }

    private function fixType($type)
    {
        return preg_replace(
            array('/^request-expired$/', '/^request-rejected$/'),
            array('request-expired-new', 'request-rejected-new'),
            $type
        );
    }

    /**
     * Returns the subject for the given template type
     *
     * @param string $type The email template name
     * @return string The subject
     */
    private function getSubject($type)
    {
        $subjects = array(
            'driver-request-cancelled-by-ro'  => 'Your booking has been cancelled',
            'booking-cancelled-by-ro'         => 'Your booking has been cancelled',
            'booking-cancelled-by-driver'     => 'Driver cancelled booking {{TCID}}',
            'request-cancelled-by-driver'     => 'Driver cancelled request {{TCID}}',
            'driver-request-canceled'         => 'Your request has been cancelled',
            'driver-booking-canceled'         => 'Your booking has been cancelled',
            'someone-else-accepted'           => 'Someone else accepted the driver before you',
            'request-canceled'                => 'Your request has been cancelled',
            'request-rejected'                => 'Your request has not been accepted, please try again',
            'driver-request-expired'          => 'Request {{REQUEST_ID}} has expired',
            'request-expired'                 => 'Your request {{REQUEST_ID}} has expired, please try again',
            'relocation-closed'               => 'Transfercar relocation complete',
            'expiring-request'                => 'The request {{REQUEST_ID}} will expire at {{REQUEST_EXPIRE_TIME}}',
            'driver-expiration-reminder'      => 'Your request is still being processed'
        );

        return isset($subjects[$type]) ? $subjects[$type] : str_replace('-', ' ', $type);
    }

    /**
     * Build the email data and returns result from buildTemplateData()
     *
     * @param string $type The email template name without the domain suffix
     * @param mixed $to Instance of User or OwnerUser
     *
     * @return array Template data returned by buildTemplateData()
     */
    private function build($type, $to)
    {
        $user = $to instanceof User && $to->get('is_owner') == false ? $this->request->get('Driver') : $this->listing->get('Owner');

        if ('booking-cancelled-by-driver' == $type)
        {
            return self::getTemplateData($this->request, $this->listing, $this->request->get('Driver'));
        }

        $data = $this->buildTemplateData($user);

        if ($type == 'expiring-request')
        {
            $rating = $this->request->get('Driver')->get('average_rating');
            sfLoader::loadHelpers(array('Url', 'Transfercar', 'Date', 'I18N'));
            $data = array_merge($data, array(
                'driver'        => $this->request->get('Driver')->getShortName(),
                'nb_relocs'     => (int) $this->request->get('Driver')->reloc_count(),
                'pickup_date'   => tc_format_date($this->request->get('pickup_date'), 'short'),
                'dropoff_date'  => tc_format_date($this->request->get('drop_off_date')),
                'nb_free_days'  => $this->listing->getIncludedDaysPrice() ? sprintf('%s x $%s per day', $this->request->get('nb_free_days'), $this->listing->getIncludedDaysPrice()) : sprintf('%s free days', $this->request->get('nb_free_days')),
                'nb_paid_days'  => $this->request->get('nb_paid_days'),
                'feedback'      => format_number_choice('[-Inf, 0) %r%% negative feedback |[0] no feedback |(0,+Inf] %r%% positive feedback', array('%r%' => sprintf('%0.0f', $rating * 100)), $rating),
                'member_since'  => format_date($this->request->get('Driver')->get('when_registered'), 'MMM yyyy'),
                'driver_avatar' => default_text($this->request->get('Driver')->get('logo'), '/images/user-icon-silhouette.png'),
                'accept_link'   => url_for('request/emailaccept?key=contact&id='.$this->request->getId().'&uid=contact', true),
                'reject_link'   => url_for('request/emailreject?key=contact&id='.$this->request->getId().'&uid=contact', true),
                'request_expire_time' => (new \DateTime($this->request->get('created_at')))->format('H:i a (T)')
            ));
        }

        if (in_array($type, array('request-expired', 'request-rejected'))) {
            $data['relocations'] = $this->getRelocations();
        }

        if ('request-expired' == $type && !$this->request->get('is_resubmit')) {
            $url = sprintf(
                '/request/confirm/pickup_date/%s/dropoff_date/%s/pickup_time/%s/dropoff_time/%s/from/Christchurch_Airport/to/Marlborough_Airport_Blenheim/relo_id/%s/resubmit/1',
                date('Y-m-d', strtotime('+1 day', strtotime($this->request->get('pickup_date')))), // add 1 day
                date('Y-m-d', strtotime('+1 day', strtotime($this->request->get('drop_off_date')))), // add 1 day
                $this->request->get('pickup_time'),
                $this->request->get('dropoff_time'),
                $this->request->get('listing_id')
            );
            $data['resubmit_link'] = $this->getTransfercarBaseURI().$url;
        }

        if ('booking-cancelled-by-driver' == $type) {
            $url = '/listing/copy/id/'.$this->request->get('listing_id');
            $data['relist_link'] = str_replace('www.', 'ro.', $this->getTransfercarBaseURI()).$url;
        }

        if ($type == 'driver-expiration-reminder') {
            $data = $this->addSuggestedListings($data);
        }

        return $data;
    }

    private function addSuggestedListings($data)
    {
        $finder = new SuggestedListings();
        $listings = $finder->find([
            'limit' => 3,
            'fromDepartures' => $this->listing->get('PickupLocation')->getDepartureNames(),
            'toDepartures' => $this->listing->get('DropoffLocation')->getDepartureNames()
        ], $this->listing->getCountryDomain());

        foreach ($listings as $i => $listing) {
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_LINK'] = $listing->getRelocationQuery('listing/view');
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_TITLE'] = $listing->get('pickupLocation')['name'] . ' to '.$listing->get('dropoffLocation')['name'];
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_DATES'] = $listing->get('from_date') . ' to '.$listing->get('to_date');
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_TYPE'] = $listing->get('vehicleType')['type'];
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_DESC'] = $listing->getDaysAllowedAvailable(). 'free days, '.$listing->get('vehicleType')['transmission'];
            $data['SUGGESTED_VEHICLE_'.($i + 1).'_IMAGE'] = $listing->get('vehicleType')['image'];
        }

        return $data;
    }

    private function getRelocations()
    {
        sfLoader::loadHelpers(array("Helper","Tag", "Url", "Transfercar", "I18N"));
        $similar = Doctrine::getTable('Listing')
            ->createQuery('l')
            ->where('l.status = ?', Listing::STATUS_NEW)
            ->addWhere('l.pickup_location_id = ?', $this->listing->get('pickup_location_id'))
            ->addWhere('l.dropoff_location_id = ?', $this->listing->get('dropoff_location_id'))
            ->addWhere('l.id != ?', $this->listing->getId())
            ->limit(3)
            ->execute();
        $site = $this->getTransfercarBaseURI();
        $data = array();
        foreach ($similar as $listing) {
            $data[] = array(
                'route_title'  => $listing->getTitle(),
                'image'        => $site . tc_thumbnail_path($listing->getVehicle()->icon_path(), 'vehicle_email'),
                'link'         => $site . $this->getRelocationPath($listing),
                'nb_days'      => $days,
                'from_date'    => tc_format_date($listing->get('date_available_from'), 'short'),
                'to_date'      => tc_format_date($listing->get('date_return_by'), 'short'),
                'vehicle_type' => $listing->get('VehicleType')->getName()
            );
        }

        return $data;
    }


    public function getRelocationPath($listing){
        $pattern = '/[^a-zA-Z0-9]/';
        $from = preg_replace($pattern, '_', $listing->PickupLocation->name);
        $to = preg_replace($pattern, '_', $listing->DropoffLocation->name);
        //$path = 'relocation/from='.$from.'&to='.$to.'&relo_id='. $listing->id;
        $path = '/relocation/'.$from.'/'.$to.'/'. $listing->id.".html";
        return $path;
    }

    /**
     * Get recipient params from the current user
     *
     * If the user in a RO (User or OwnerUser), then the recipient data is retrieve from
     * the listing data, using either OwnerContacts, Manager or RO emails params.
     *
     * @param mixed $to Instance of User or OwnerUser
     *
     * @return array Array containing email and name of the recipient(s)
     */
    private function getRecipient($to)
    {
        // driver
        if ($to instanceof User && $to->get('is_owner') == false)
        {
            return $to->getEmailToParams();
        }
        else
        {
            return $this->listing->getNewRequestContact()->getEmailToParams();
        }
    }

    /**
     * Build the template data from the current listing and request
     *
     * @param User $to Instance of User
     *
     * @return array Array containing email and name of the recipient(s)
     */
    private function buildTemplateData($user)
    {
        sfLoader::loadHelpers('Date', 'Transfercar');
        $template_data = array(
            'firstname'       => $user->firstname,
            'lastname'        => $user->lastname,
            'username'        => $user->username,
            'when_registered' => $user->when_registered,
            'request_id'      => $this->request->getIncrementedId(),
            'tcid'            => $this->request->get('listing_id'),
            'from'            => $this->listing->PickupLocation->name,
            'from_date'       => format_date($this->request->get('pickup_date'), 'd', 'en_'.strtoupper($this->domain)),
            'pickup_date'     => format_date($this->request->get('pickup_date'), 'd', 'en_'.strtoupper($this->domain)),
            'to'              => $this->listing->DropoffLocation->name,
            'to_date'         => format_date($this->request->get('drop_off_date'), 'd', 'en_'.strtoupper($this->domain)),
            'dropoff_date'    => format_date($this->request->get('drop_off_date'), 'd', 'en_'.strtoupper($this->domain)),
            'vehicle_type'    => $this->listing->get('VehicleType')->get('description'),
            'email_opt_link'  => $this->getTransfercarBaseURI().'/userSettings/emailOptions',
            'no_show_fee'        => $this->request->rawGet('credit_card_id') ? 1 : 0,
            'no_show_fee_amount' => $this->request->get('no_show_fee_amount')
        );

        if ($user->get('is_owner'))
        {
            $template_data = $this->addROData($template_data);
        }
        else
        {
            $template_data = $this->addDriverData($template_data);
        }

        $template_data = array_merge(
            $template_data,
            $this->listing->detailedInfoArray()
        );


        return $template_data;
    }

    /**
     * Adds RO specific data
     *
     * @param array $template_data initial template data
     *
     * @return array Template data
     */
    private function addROData($template_data)
    {
        $groupedRequests = $this->groupedRequests ? $this->groupedRequests : array($this->request);
        $template_data['driver']         = $this->request->get('Driver')->get('username');
        $template_data['reloc_details']  = $this->request->buildRelocLocationDetails($groupedRequests);

        return $template_data;
    }

    /**
     * Adds Driver specific data
     *
     * @param array $template_data initial template data
     *
     * @return array Template data
     */
    private function addDriverData($template_data)
    {
        $template_data['reloc_details'] = $this->listing->detailedInfo(true, true);
        $template_data['owner_details'] = $this->listing->Owner->detailedInfo();
        $template_data['owner_username'] = $this->listing->Owner->get('username');
        $template_data['owner_company'] = $this->listing->Owner->get('company');

        // special handling for rated, adding rating details
        if ($this->request->isStatus(Request::STATUS_RATED))
        {
            switch ($this->request->rating) {
              case -1:
                $template_data['rate'] = 'negative';
                break;

              case 0:
                $template_data['rate'] = 'neutral';
                break;

              case 1:
                $template_data['rate'] = 'positive';
                break;
            }

        }

        // special handling for rated and manually cancelled requests, adding comment
        if ($this->request->isStatus(Request::STATUS_RATED, Request::STATUS_CANCELED_BY_DRIVER, Request::STATUS_DRIVER_CANCELED, Request::STATUS_OWNER_CANCELED))
        {
            $template_data['comment'] = $this->request->rating_comment;
        }

        // special handling for canceled requests, not disclosing RO details
        if ($this->request->isStatus(Request::STATUS_CANCELED))
        {
            $template_data['reloc_details'] = $this->listing->detailedInfo(false); // not show owner data
            unset($template_data['owner_details']); // not show owner details
        }

        return $template_data;
    }

    private function getTransfercarBaseURI()
    {
        $site = sfConfig::get('app_domain_' . $this->domain);
        $protocol = strpos($site, 'staging') === 0 ? 'http' : 'https';
        return $protocol.'://' . $site;
    }
}

