<?php

namespace Transfercar\Ranking;

use Listing;
use Settings;
use Transfercar\Geoloc\GoogleMapsDistancesMatrix;

class ListingVoter implements VoterInterface
{
    const PAID_DAYS_WEIGHT = 0.5;

    private $debug = [];
    private $errors = [];

    /**
     * Calculate the score for the given Listing
     *
     * @see https://basecamp.com/1780292/projects/11507933/todos/230906429#comment_429251496
     * @param Listing $listing
     * @return float The Listing score
     */
    public function getUpVotes($listing)
    {
        $includedScore = $this->getExtraIncludedScore($listing);
        $freeDaysScore = $this->getFreeDaysScore($listing);
        $paidDaysScore = $this->getPaidDaysScore($listing);

        $score = ($includedScore + $freeDaysScore + $paidDaysScore);

        $this->debug[] = sprintf('Listing upvotes = 10 * (included + free_days + paid_days) = 10 * (%s + %s + %s) = %s',
            round($includedScore, 2),
            round($freeDaysScore, 2),
            $paidDaysScore,
            round($score * 10, 2));

        return $score;
    }

    public function getDownVotes($listing)
    {
        $score = (1 - $this->getExtraIncludedScore($listing)) / 10
            +  max(0, 1 - $this->getFreeDaysScore($listing));

        $this->debug[] = sprintf('Listing downvotes = %s', round($score, 2));

        return $score;
    }

    public function getErrors()
    {
        $errorMessages = [
            'included_extras' => 'Allow more included extras',
            'days_allowed'    => 'Days allowed are less than recommended by Transfercar',
            'paid_days_included' => 'Allow some optional paid days'
        ];

        return array_intersect_key($errorMessages, $this->errors);
    }

    public function getDebug()
    {
        return implode("\n", $this->debug);
    }

    private function getExtraIncludedScore(Listing $listing)
    {
        // insurance is not counted as mandatory
        $nb_included  = (int) $listing->get('petrol_included')
             + (int) $listing->get('ferry_included')
             + (int) $listing->get('expenses_included')
             + (int) $listing->get('tolls_included');

        $max_included = $listing->getCountryDomain() == 'nz' ? 3 : 4; // no road tolls in NZ

        if ($max_included > $nb_included) {
            $this->errors['included_extras'] = true;
        }

        return $nb_included / $max_included;
    }

    private function getFreeDaysScore(Listing $listing)
    {
        $nb_free_days = $listing->getDaysAllowedAvailable();

        if (!$recommended_days = $this->getRecommendedDays($listing)) {
            return 0;
        }

        $extra_free_days = $nb_free_days - $recommended_days;

        if ($recommended_days > $nb_free_days) {
            $this->errors['days_allowed'] = true;
        }

        return $extra_free_days / $recommended_days;
    }

    private function getPaidDaysScore(Listing $listing)
    {
        if (!$listing->get('paid_days_allowed')) {
            $this->errors['paid_days_included'] = true;
        }

        return self::PAID_DAYS_WEIGHT * ($listing->get('paid_days_allowed') ? 1 : 0);
    }

    private function getRecommendedDays(Listing $listing)
    {
        $api      = new GoogleMapsDistancesMatrix();
        $distance = $api->getDistances(
            $listing->get('PickupLocation')->getNameWithCountry('name'),
            $listing->get('DropoffLocation')->getNameWithCountry('name'),
            $listing->getCountryDomain() == 'us' ? 'imperial' : 'metric'
        );

        if (false === $distance)
        {
            $distance = $listing->getApproximateDistance();
        }

        return ceil($distance/Settings::getDistanceByDay($listing->getCountryDomain()));
    }

}